package com.willalp.web.controller.app;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.willalp.canteen.domain.*;
import com.willalp.canteen.domain.req.GiftCustomPayReq;
import com.willalp.canteen.domain.vo.UserShopCarJsonDoMainVo;
import com.willalp.canteen.service.*;
import com.willalp.clockingin.domain.BizLeave;
import com.willalp.clockingin.service.IBizLeaveService;
import com.willalp.common.core.controller.BaseController;
import com.willalp.common.core.domain.AjaxResult;
import com.willalp.common.core.domain.entity.SysUser;
import com.willalp.common.core.page.TableDataInfo;
import com.willalp.common.enums.SysOrgCodeEnum;
import com.willalp.common.exception.base.BaseException;
import com.willalp.common.utils.DateUtils;
import com.willalp.common.utils.StringUtils;
import com.willalp.common.utils.bean.BeanUtils;
import com.willalp.common.utils.http.HttpUtils;
import com.willalp.device.domain.HsDeviceRepairs;
import com.willalp.device.service.IHsDeviceRepairsService;
import com.willalp.event.domain.HsEvent;
import com.willalp.event.domain.HsEventBusinessRelation;
import com.willalp.event.service.IHsEventBusinessRelationService;
import com.willalp.event.service.IHsEventService;
import com.willalp.flow.domain.HsFlowBase;
import com.willalp.flow.domain.HsFlowRunningRecord;
import com.willalp.flow.domain.HsFlowTask;
import com.willalp.flow.service.HsFlowEngineService;
import com.willalp.flow.service.IHsFlowBaseService;
import com.willalp.flow.service.IHsFlowRunningRecordService;
import com.willalp.flow.service.IHsFlowTaskService;
import com.willalp.msg.domain.HsAppUserMsg;
import com.willalp.msg.service.IHsAppUserMsgService;
import com.willalp.push.service.IHsAppPushService;
import com.willalp.system.domain.SysUserOrganization;
import com.willalp.system.domain.req.AppSysReq;
import com.willalp.system.service.IHsPostConfigService;
import com.willalp.system.service.ISysConfigService;
import com.willalp.system.service.ISysUserOrganizationService;
import com.willalp.system.service.ISysUserService;
import com.willalp.wx.domain.WxMessageBeanApprovalPass;
import com.willalp.wx.domain.WxMessageBeanApprovalShop;
import com.willalp.wx.domain.WxMsgTemplateInExcSuccess;
import com.willalp.wx.enums.WxTemplateKeyEnum;
import com.willalp.wx.service.WxMsgService;
import com.willalp.wx.utils.WxUtils;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Api("app接口")
@RestController
@RequestMapping("/app")
public class AppTwoController extends BaseController {

    @Resource
    private IHsPostConfigService hsPostConfigService;

    @Resource
    private IHsAppUserMsgService hsAppUserMsgService;

    @Resource
    private IHsFlowTaskService taskService;

    @Resource
    private HsFlowEngineService flowEngineService;

    @Resource
    private IBizLeaveService bizLeaveService;

    /**
     * 当前流程审批人员
     * @param groupId
     * @param baseId
     */
    @GetMapping("/currentPL")
    public AjaxResult currentPL(String taskId, String groupId, String baseId) {
        return AjaxResult.success(flowEngineService.getApprovalTreePost(taskId, groupId, baseId));
    }

    /**
     * 当前流程下一级审批人员
     * @param groupId
     * @param baseId
     */
    @GetMapping("/pL")
    public AjaxResult list(String taskId, String groupId, String baseId) {
        return AjaxResult.success(flowEngineService.getNextApprovalTreePost(taskId, groupId, baseId));
    }

    /**
     * 查询岗位配置列表
     * @param taskId
     * @param candidate
     */
    @GetMapping("/task/submit/candidate")
    public AjaxResult submitCandidate(String taskId, String candidate, String applyUserName) {
        return AjaxResult.success(flowEngineService.submitCandidate(taskId, candidate, applyUserName));
    }


    /**
     * 获取消息列表
     *
     * @param request
     * @return
     */
    @RequestMapping("/gum")
    public TableDataInfo getUserMsg(HttpServletRequest request) {
        startPage();
        return getDataTable(
                hsAppUserMsgService.getBaseMapper().selectList(new QueryWrapper<HsAppUserMsg>()
                        .eq("user_code", request.getHeader("UserName"))
                        .last("order by msg_is_read asc,create_time desc")));
    }

    /**
     * 更新消息已读
     *
     * @param uc
     * @param request
     * @return
     */
    @RequestMapping("/rm")
    public AjaxResult readMsg(String id, String uc, HttpServletRequest request) {
        hsAppUserMsgService.readMsgById(id);
        hsAppUserMsgService.allReadMsgByUserCode(uc);
        return AjaxResult.success();
    }

    /**
     * 更新审批任务已读
     *
     * @param taskId
     * @return
     */
    @RequestMapping("/rt")
    public AjaxResult readTask(String taskId, HttpServletRequest request) {
        taskService.readTask(taskId);
        String userName = request.getHeader("UserName");
        hsAppUserMsgService.readMsgByTaskId(taskId, userName);
        hsAppUserMsgService.readReceiptVerify(userName);
        return AjaxResult.success(taskId);
    }

    /**
     * 更新我的发起已读
     *
     * @param id
     * @return
     */
    @RequestMapping("/rl")
    public AjaxResult readTask(String id) {
        if (StringUtils.isNotBlank(id)) {
            BizLeave byId = bizLeaveService.getById(id);
            if (byId != null) {
                byId.setRemark("");
                bizLeaveService.updateById(byId);
            }
        }
        return AjaxResult.success();
    }

    /**
     * 查询消息总信息
     *
     * @param request
     * @return
     */
    @RequestMapping("/msgs")
    public AjaxResult msgSum(HttpServletRequest request) {
//        return AjaxResult.success(hsAppUserMsgService.pushMsgSum(request.getHeader("UserName")));
        return AjaxResult.success();
    }

    @Resource
    private ISysUserOrganizationService sysUserOrganizationService;

    @Autowired
    private IHsGiftService hsGiftService;

    @Resource
    private IHsGiftOrderService giftOrderService;

    @Resource
    private IHsGiftOrderDetailService giftOrderDetailServic;

    @Resource
    private WxUtils wxUtils;

    /**
     * 获取积分记录
     *
     * @param uc
     * @return
     */
    @RequestMapping("/ac")
    @ResponseBody
    public AjaxResult getAc(@RequestParam String uc) {
        if (StringUtils.isBlank(uc)) {
            return AjaxResult.error("用户系统编码不能为空");
        }
        return AjaxResult.success(sysUserOrganizationService.getBaseMapper().selectOne(new QueryWrapper<SysUserOrganization>()
                .eq("organization_code", SysOrgCodeEnum.HuiJiFen.getCode())
                .eq("login_name", uc)).getIntegral());
    }

    /**
     * 积分商城礼品列表
     *
     * @param req
     * @return
     */
    @RequestMapping("/getGift")
    @ResponseBody
    public TableDataInfo getGift(@RequestBody AppSysReq req) {
        startPage();
        List<HsGift> hsGifts = hsGiftService.appSelectedHsGiftList(req.getGiftType(), req.getGiftTypeCode());
        return getDataTable(hsGifts);
    }

    @Resource
    private IHsGiftOrderDetailService giftOrderDetailService;

    /**
     * 获取订单信息详情
     *
     * @param sys
     * @return
     */
    @RequestMapping("/getGiftOrderDetail")
    @ResponseBody
    public TableDataInfo getOrderDetail(@RequestBody AppSysReq sys) {
        startPage();
        HsGiftOrderDetail hcod = new HsGiftOrderDetail();
        hcod.setOrderNumber(sys.getOrderNumber());
        return getDataTable(giftOrderDetailService.selectHsCanteenOrderDetailList(hcod));
    }

    /**
     * 退单
     *
     * @param on
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @RequestMapping("backOd")
    @ResponseBody
    public AjaxResult backOrder(String on, HttpServletRequest request) {
        try {
            if (StringUtils.isBlank(on)) {
                return AjaxResult.error("单号不可为空");
            }
            String userName = request.getHeader("UserName");
            if (StringUtils.isBlank(userName)) {
                return AjaxResult.error("非法接口请求 - 用户无法获取");
            }
            //获取订单信息
            HsGiftOrder one = giftOrderService.getOne(new QueryWrapper<HsGiftOrder>()
                    .eq("order_number", on)
                    .eq("user_number", userName));
            if (null != one) {
                if (!"0".equals(one.getChargebackSign())) {
                    return AjaxResult.error("您已完成退单");
                }
                HsGiftOrderDetail hsGiftOrderDetail = giftOrderDetailServic.getBaseMapper().selectOne(new QueryWrapper<HsGiftOrderDetail>()
                        .eq("order_number", one.getOrderNumber())
                        .eq("is_delete", 0)
                        .last("limit 1"));
                if (hsGiftOrderDetail == null) {
                    return AjaxResult.error("无法查询到订单详情，请联系管理员");
                }
                String eventId = hsGiftOrderDetail.getEventId();
                if (StringUtils.isNotBlank(eventId)) {
                    HsEvent event = eventService.getById(eventId);
                    if (event != null) {
                        Date eventEndTime = event.getEventEndTime();
                        if (eventEndTime != null && eventEndTime.getTime() < System.currentTimeMillis()) {
                            return AjaxResult.error("订单已过可退时间");
                        }
                    }
                }
                HsGift hsGift = hsGiftService.getOne(new QueryWrapper<HsGift>()
                        .eq("gift_number", hsGiftOrderDetail.getGiftNumber())
                        .eq("is_delete", 0)
                        .last("limit 1"));
                hsGift.setAmount(hsGiftOrderDetail.getAmount() + hsGiftOrderDetail.getAmount());
                one.setChargebackSign("1");
                BigDecimal orderTotal = one.getOrderTotal();
                if (null != orderTotal) {
                    //积分退回
                    SysUserOrganization oneSysUo = sysUserOrganizationService.getOne(new QueryWrapper<SysUserOrganization>()
                            .eq("organization_code", "G00000007")
                            .eq("login_name", userName));
                    oneSysUo.setIntegral(oneSysUo.getIntegral().add(orderTotal));
                    sysUserOrganizationService.updateById(oneSysUo);
                }
                //生成记录
//                HsAmassScoreChange asc = new HsAmassScoreChange();
//                Date date = new Date();
//                asc.setCreateTime(date);
//                asc.setChangeDate(date);
//                asc.setChangeType("3");
//                asc.setAmassScore(orderTotal);
//                asc.setUserCode(userName);
//                hsAmassScoreChangeService.save(asc);
//                //生成进度记录
//                HsOrderProcess hop = new HsOrderProcess();
//                hop.setOrderNo(on);
//                hop.setProcessStatus("已完成");
//                hop.setProcessDesc("积分已退还，订单结束");
//                hop.setProcessDate(new Date());
//                hop.setUserCode(userName);
//                orderProcessService.save(hop);
                //修改订单状态
                one.setStatusText("已退单");
                one.setIsNews("1");
                hsGiftService.updateById(hsGift);
                giftOrderService.updateById(one);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return AjaxResult.error();
        }
        return AjaxResult.success();
    }


    @Resource
    private HttpUtils httpUtils;

    @Resource
    private IHsAppUserMsgService userMsgService;

    @Resource
    private IHsOrderProcessService orderProcessService;

    @Resource
    private WxMsgService wxMsgService;

    @Resource
    private IHsEventService eventService;

    @Resource
    private IHsEventBusinessRelationService eventBusinessRelationService;

    /**
     * 积分商城下单
     *
     * @param req
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @RequestMapping("giftOrder")
    @ResponseBody
    public AjaxResult giftOrder(@RequestBody GiftCustomPayReq req) {
        //验证用户
        SysUserOrganization sysUserOrganization = sysUserOrganizationService.getBaseMapper().selectOne(new QueryWrapper<SysUserOrganization>()
                .eq("login_name", req.getUserNumber()).eq("organization_code", req.getJgid()));
        if (null == sysUserOrganization) {
            return AjaxResult.error("用户信息不存在");
        }
        if (sysUserOrganization.getIntegral().compareTo(req.getOrderTotal()) < 0) {
            return AjaxResult.error("账户积分不足");
        }
        //生成微信推送模板
        WxMsgTemplateInExcSuccess wxMsgTemplateInExcSuccess = new WxMsgTemplateInExcSuccess();
        //生成审批结果消息列表
        HsAppUserMsg appUserMsg = new HsAppUserMsg();
        appUserMsg.setMsgTitle("积分兑换成功");
        appUserMsg.setUserCode(getUsername());
        appUserMsg.setMsgIsRead("0");
        appUserMsg.setCreateTime(new Date());
        appUserMsg.setMsgType("6");
        StringBuilder content = new StringBuilder();
        //消费标志
        HsGiftOrder hsGiftOrder = new HsGiftOrder();
        hsGiftOrder.setConsumeSign(0);
        hsGiftOrder.setZfbz(req.getOrderType());
        hsGiftOrder.setUserNumber(req.getUserNumber());
        hsGiftOrder.setUserName(req.getUserName());
        hsGiftOrder.setPhone(req.getPhone());
        hsGiftOrder.setPlaceNumber(req.getPlaceNumber());
        hsGiftOrder.setStatusText("已下单");
        //订单
        BeanUtils.copyBeanProp(hsGiftOrder, req);
        //获取id
        hsGiftOrder.setOrderId(IdWorker.getIdStr());
        Date date = new Date();
        hsGiftOrder.setPaytime(date);
        wxMsgTemplateInExcSuccess.setTime5(wxUtils.getFormat(DateUtils.parseDateToStr("yyyy-MM-dd HH:mm:ss", date)));
        //订单详情
        if (req.getHsGifts().size() == 0) {
            return AjaxResult.error("订单明细不能为空");
        }
        List<HsEventBusinessRelation> hsEventBusinessRelations = null;
        if (req.getHsGifts().get(0) != null && StringUtils.isNotBlank(req.getHsGifts().get(0).getEventId())) {
            hsEventBusinessRelations = eventBusinessRelationService.getBaseMapper().selectList(new QueryWrapper<HsEventBusinessRelation>()
                    .eq("event_id", req.getHsGifts().get(0).getEventId()));
        } else {
            return AjaxResult.error("活动配置出错");
        }
        List<HsGift> hsGifts = new ArrayList<>();
        StringBuilder nameStr = new StringBuilder();
        //获取订单编码
//        String data = httpUtils.sendGet(httpUtils.HTTPS + httpUtils.host + ":" + httpUtils.port + httpUtils.path + "/get/onb");
        String orderNumber = "CS" + IdWorker.getIdStr();
        hsGiftOrder.setOrderNumber(orderNumber);
        appUserMsg.setRemark(orderNumber);
        //库存检查
        for (HsGiftOrderDetail gift : req.getHsGifts()) {
            String giftId = gift.getGiftId();
            HsGift hsGift = hsGiftService.getById(giftId);
            if (hsGift == null) {
                return AjaxResult.error("未找到物品信息下单失败");
            }
            gift.setPrice(hsGift.getGiftPrice());
            List<HsEventBusinessRelation> collect = hsEventBusinessRelations.stream().filter(r -> StringUtils.isNotBlank(r.getBusinessId()) &&
                    gift.getGiftId().equals(r.getBusinessId())).collect(Collectors.toList());
            if (collect.size() > 0) {
                if (collect.get(0).getLimitNum() < gift.getAmount()) {
                    return AjaxResult.error("【" + gift.getGiftName() + "】只能兑换" + collect.get(0).getLimitNum() + "个");
                }
            }
            //如果是活动商品
            if (StringUtils.isNotBlank(gift.getEventId())) {
                //获取结束时间
                HsEvent eventbyId = eventService.getById(gift.getEventId());
                long time = eventbyId.getEventEndTime().getTime();
                //当前时间
                long l = System.currentTimeMillis();
                //结束时间大于当前时间（活动范围内），则支持下单
                if (time < l) {
                    return AjaxResult.error("活动已结束");
                }
            }
            //抵扣库存
            Integer amount = gift.getAmount();
            if (hsGiftService.checkAndReduceGiftInventory(amount, giftId) < 0) {
                return AjaxResult.error(gift.getGiftName() + "库存不足，下单失败");
            }
            nameStr.append("【").append(gift.getGiftName()).append("】");
            gift.setDetailId(IdWorker.getIdStr());
            gift.setOrderNumber(orderNumber);
        }
        //消费积分
        BigDecimal integral = sysUserOrganization.getIntegral();
        hsGiftOrder.setRemark(integral.toString() + " > ");
        BigDecimal subtract = integral.subtract(req.getOrderTotal());
        hsGiftOrder.setRemark(hsGiftOrder.getRemark() + subtract);
        sysUserOrganization.setIntegral(subtract);
        try {
            //删除购物车
            for (HsGiftOrderDetail o : req.getHsGifts()) {
                userShopCarService.deleteJson(o.getGiftId(), req.getUserNumber());
            }
            //更新DB数据
            sysUserOrganizationService.updateById(sysUserOrganization);//更新账户信息
            giftOrderService.save(hsGiftOrder);//订单主表
            giftOrderDetailServic.saveBatch(req.getHsGifts());//订单明细
            //发送微信订阅通知
            wxMsgTemplateInExcSuccess.setThing3(wxUtils.getFormat(String.valueOf(req.getOrderTotal())));
            wxMsgTemplateInExcSuccess.setThing6(wxUtils.getFormat(String.valueOf(sysUserOrganization.getIntegral())));
            wxMsgTemplateInExcSuccess.setThing4(wxUtils.getFormat(nameStr.toString()));
            content.append("<p>兑换物品：").append(nameStr).append("</p>")
                    .append("<p>抵扣积分：").append(req.getOrderTotal()).append("</p>")
                    .append("<p>兑换时间：").append(DateUtils.parseDateToStr("yyyy-MM-dd HH:mm:ss", date)).append("</p>");
            String wxMsgContent = "兑换物品：" + nameStr;
            WxMessageBeanApprovalShop wx = new WxMessageBeanApprovalShop(DateUtils.getTime(), wxMsgContent);
            appUserMsg.setMsgContent(content.toString());
            String openid = pushService.getOpenIdByUserNo(getUsername());
            wxMsgService.sendQRCodeMsg(openid, wx, WxTemplateKeyEnum.INTEGRAL_EXCHANGE_SUCCESS);
            hsGiftOrder.setRemark(nameStr.toString());
            //生成订单已下单进度
            List<HsOrderProcess> opList = new ArrayList<>();
            HsOrderProcess op = new HsOrderProcess();
            op.setOrderNo(orderNumber);
            op.setProcessNum(1L);
            op.setUserCode(req.getUserNumber());
            op.setProcessDesc("已下单");
            op.setProcessDate(new Date());
            opList.add(op);
            //记录积分变动
            //HsAmassScoreChange hasc = new HsAmassScoreChange();
            //hasc.setChangeDate(new Date());
            //hasc.setAmassScore(req.getOrderTotal());
            //hasc.setChangeType("2");
            //hasc.setUserCode(req.getUserNumber());
            userMsgService.save(appUserMsg);
            //hsAmassScoreChangeService.save(hasc);
            //orderProcessService.saveBatch(opList);
        } catch (Exception e) {
            e.printStackTrace();
            return AjaxResult.error("下单失败");
        }
        return AjaxResult.success();
    }

    @Resource
    private IHsGiftTypeService giftTypeService;

    @Resource
    private ISysConfigService sysConfigService;

    /**
     * 获取订单详情
     *
     * @return
     */
    @RequestMapping("/orderInfo")
    @ResponseBody
    public AjaxResult getOrderInfo(String on) {
        HsGiftOrder hco = new HsGiftOrder();
        hco.setOrderNumber(on);
        List<HsGiftOrder> hsGiftOrders = giftOrderService.selectHsCanteenOrderList(hco);
        if (hsGiftOrders == null) {
            return AjaxResult.error();
        }
        return AjaxResult.success(hsGiftOrders.get(0));
    }

    /**
     * 礼品类型
     *
     * @param request
     * @return
     */
    @RequestMapping("/giftType")
    public AjaxResult appSelectHsGiftType(HttpServletRequest request) {
        return AjaxResult.success(giftTypeService.appSelectHsGiftType());
    }

    /**
     * 礼品登记公告
     */
    @GetMapping("/getNotice")
    public AjaxResult getNotice() {
        return AjaxResult.success(sysConfigService.selectConfigByKey("noticeText"));
    }

    /**
     * 公告状态
     */
    @GetMapping("/getNoticeStatus")
    public AjaxResult getNoticeStatus() {
        return AjaxResult.success(sysConfigService.selectConfigByKey("noticeStatus"));
    }

    @Resource
    private IHsUserShopCarService userShopCarService;

    /**
     * 根据用户查询购物车
     *
     * @param request
     * @return
     */
    @RequestMapping("/getUsc")
    public AjaxResult queryShopCarByUser(HttpServletRequest request) {
        return AjaxResult.success(userShopCarService.queryShopCarByUser(request.getHeader("UserName")));
    }

    /**
     * 加入购物车
     *
     * @param userShopCarJsonDoMainVo
     * @return
     */
    @RequestMapping("/addShopCar")
    public AjaxResult addShopCar(@RequestBody UserShopCarJsonDoMainVo userShopCarJsonDoMainVo) {
        return AjaxResult.success(userShopCarService.jsonAdd(userShopCarJsonDoMainVo));
    }

    /**
     * 购物车物品总数
     *
     * @param request
     * @return
     */
    @RequestMapping("/getSCN")
    public AjaxResult getShopCarNum(HttpServletRequest request) {
        return AjaxResult.success(userShopCarService.getShopCarNum(request.getHeader("UserName")));
    }

    @Resource
    private IHsCanteenDiningPlaceService diningPlaceService;

    @Resource
    private IHsFlowRunningRecordService runningRecordService;

    @Resource
    private IHsFlowBaseService flowBaseService;

    @Resource
    private ISysUserService sysUserService;

    @Resource
    private IHsAppPushService pushService;


    @Transactional(rollbackFor = Exception.class)
    @RequestMapping("/useSealEnd")
    public AjaxResult useSealEnd(String taskId) {
        if (StringUtils.isBlank(taskId)) {
            return AjaxResult.error();
        }
        HsFlowTask flowTask = taskService.getById(taskId);
        if (null == flowTask) {
            return AjaxResult.error();
        }
        if (!getUsername().equals(flowTask.getOperator())) {
            return AjaxResult.error("没有操作权限");
        }
        if ("1".equals(flowTask.getIsEnd())) {
            return AjaxResult.error("任务已结束，无法操作");
        }
        flowTask.setIsEnd("1");
        HsFlowRunningRecord runningRecordEnd = new HsFlowRunningRecord();
        runningRecordEnd.setTaskId(taskId);
        runningRecordEnd.setYesOrNo("RESULT_TRUE_END");
        runningRecordEnd.setComment(flowTask.getBaseName() + "：" + getLoginUser().getUser().getNickName() + " - 同意并结束");
        runningRecordEnd.setOperator(getUsername());
        runningRecordService.save(runningRecordEnd);
        taskService.updateById(flowTask);

        //生成审批结果消息列表
        HsAppUserMsg appUserMsg = new HsAppUserMsg();
        appUserMsg.setMsgContent("您的 [" + flowTask.getTaskName() + "] 申请已通过");
        appUserMsg.setMsgTitle("审批结果");
        appUserMsg.setMsgIsRead(HsFlowEngineService.ZERO);
        appUserMsg.setUserCode(flowTask.getInitiator());
        appUserMsg.setMsgType(HsFlowEngineService.FOUR);
        appUserMsg.setIsPush(HsFlowEngineService.ZERO);
        appUserMsg.setRemark(flowTask.getId());
        appUserMsg.setCreateTime(new Date());
        userMsgService.save(appUserMsg);
        SysUser initiatorInfo = sysUserService.selectUserByUserName(flowTask.getInitiator());
        String openid = pushService.getOpenIdByUserNo(flowTask.getInitiator());
        WxMessageBeanApprovalPass wxMessageBeanApprovalWait = new WxMessageBeanApprovalPass(
                flowTask.getTaskName() + HsFlowEngineService.APPROVED, initiatorInfo.getNickName(), DateUtils.getTime());
        wxMsgService.sendQRCodeMsg(openid, wxMessageBeanApprovalWait, WxTemplateKeyEnum.APPROVAL_RESULT);

        HsFlowBase node_813096256777 = flowBaseService.getFlowTaskBaseByGroupIdAndBaseId("9902a47f-f24b-4010-8889-17a1dfb9b057",
                "node_813096256777");
        List<HsFlowTask> tasks = new ArrayList<>();
        tasks.add(flowTask);
        List<HsFlowTask> hsFlowTasks = flowEngineService.getHsFlowTasks(tasks);
        if (hsFlowTasks.size() > 0) {
            flowEngineService.baseRun(node_813096256777, flowTask, hsFlowTasks.get(0).getFormData());
        }
        return AjaxResult.success();
    }

    @Resource
    private IHsDeviceRepairsService deviceRepairsService;

    @RequestMapping("/submitMaintenanceRecord")
    public AjaxResult submitMaintenanceRecord(@RequestBody HsDeviceRepairs deviceRepairs) {
        String id = deviceRepairs.getId();
        String maintenanceRecord = deviceRepairs.getMaintenanceRecord();
        if (StringUtils.isBlank(id) && StringUtils.isBlank(maintenanceRecord)) {
            return AjaxResult.error();
        }
        HsDeviceRepairs repairs = deviceRepairsService.getById(id);
        repairs.setMaintenanceRecord(maintenanceRecord);
        return toAjax(deviceRepairsService.updateById(repairs));
    }
}
