package com.willalp.web.controller.clockingin;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.willalp.common.annotation.Log;
import com.willalp.common.core.controller.BaseController;
import com.willalp.common.core.domain.AjaxResult;
import com.willalp.common.core.page.TableDataInfo;
import com.willalp.common.enums.BusinessType;
import com.willalp.common.utils.DateUtils;
import com.willalp.common.utils.SecurityUtils;
import com.willalp.common.utils.StringUtils;
import com.willalp.common.utils.bean.BeanUtils;
import com.willalp.common.utils.poi.ExcelUtil;
import com.willalp.flow.domain.*;
import com.willalp.flow.domain.vo.FileApprovalVo;
import com.willalp.flow.service.HsFlowEngineService;
import com.willalp.flow.service.IHsLetterDisposeService;
import com.willalp.flow.service.IFileApprovalService;
import com.willalp.flow.service.IHsContractManageService;
import com.willalp.msg.service.IHsAppUserMsgService;
import com.willalp.system.service.ISysConfigService;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 文件审批业务Controller
 *
 * @author willalp
 * @date 2022-11-30
 */
@RestController
@RequestMapping("/flow/accessory")
public class FileApprovalController extends BaseController {
    @Resource
    private IFileApprovalService fileApprovalService;

    @Resource
    private IHsContractManageService contractManageService;

    @Resource
    private IHsLetterDisposeService letterDisposeService;

    @Resource
    private HsFlowEngineService flowEngineService;

    @Resource
    private ISysConfigService sysConfigService;

    @Resource
    private IHsAppUserMsgService userMsgService;


    /**
     * 查询文件审批业务列表
     */
    @GetMapping("/list")
    public TableDataInfo list(FileApproval fileApproval) {
        startPage();
        List<FileApproval> list = fileApprovalService.selectFileApprovalList(fileApproval);
        return getDataTable(list);
    }

    /**
     * 导出文件审批业务列表
     */
    @Log(title = "文件审批业务", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(FileApproval fileApproval) {
        List<FileApproval> list = fileApprovalService.selectFileApprovalList(fileApproval);
        ExcelUtil<FileApproval> util = new ExcelUtil<FileApproval>(FileApproval.class);
        return util.exportExcel(list, "文件审批业务数据");
    }

    /**
     * 获取文件审批业务详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(fileApprovalService.getById(id));
    }

    /**
     * 新增文件审批业务
     */
    @Log(title = "文件审批业务", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody FileApprovalVo fileApprovalVo) {
        if (StringUtils.isBlank(fileApprovalVo.getSubmitType())) {
            return AjaxResult.error("非法提交");
        }
        FileApproval fileApproval;
        HsContractManage contractManage;
        HsLetterDispose letterDispose;
        String runKey = null;

        Map<String, Object> params = fileApprovalVo.getParams();

        fileApprovalVo.setId(IdWorker.getIdStr());
        fileApprovalVo.setApplyUserId(getUsername());
        fileApprovalVo.setApplyUserName(getLoginUser().getUser().getNickName());
        fileApprovalVo.setCreateBy(getUsername());
        fileApprovalVo.setCreateTime(DateUtils.getNowDate());

        String files = JSON.toJSONString(fileApprovalVo.getFileList());
        String imgs = JSON.toJSONString(fileApprovalVo.getImgList());

        if ("1".equals(fileApprovalVo.getSubmitType())) {
            fileApproval = new FileApproval();
            BeanUtils.copyProperties(fileApprovalVo, fileApproval);
            fileApproval.setFileList(files);
            fileApproval.setImgList(imgs);
            runKey = "file_approval_flow_run_key";
            fileApprovalService.save(fileApproval);
        } else if ("2".equals(fileApprovalVo.getSubmitType())) {
            contractManage = new HsContractManage();
            BeanUtils.copyProperties(fileApprovalVo, contractManage);
            contractManage.setFileList(files);
            contractManage.setImgList(imgs);
            runKey = "contract_approval_flow_run_key";
            contractManageService.save(contractManage);
        } else if ("3".equals(fileApprovalVo.getSubmitType())) {
            letterDispose = new HsLetterDispose();
            BeanUtils.copyProperties(fileApprovalVo, letterDispose);
            letterDispose.setFileList(files);
            letterDispose.setImgList(imgs);
            runKey = "letter_dispose_flow_run_key";
            letterDisposeService.save(letterDispose);
        }
        String taskId = IdWorker.getIdStr();
        flowEngineService.submitTask(new HsFlowTask(taskId, fileApprovalVo.getId(), sysConfigService.selectConfigByKey(runKey)), params);
        //异步生成审批消息列表
        String content = "<p>标题：" + fileApprovalVo.getTitle() + "</p>" +
                "<p>申请人：" + fileApprovalVo.getApplyUserName() + "</p>";
        userMsgService.saveApprovalMsgAndPush("5",
                fileApprovalVo.getTitle(), content, taskId,
                String.valueOf(params.get("candidate")), fileApprovalVo.getApplyUserName(), fileApprovalVo.getCreateTime());
        return AjaxResult.success();
    }

    /**f
     * 修改文件审批业务
     */
    @Log(title = "文件审批业务", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody FileApproval fileApproval) {
        fileApproval.setUpdateBy(SecurityUtils.getUsername());
        fileApproval.setUpdateTime(DateUtils.getNowDate());
        return toAjax(fileApprovalService.updateById(fileApproval));
    }

    /**
     * 删除文件审批业务
     */
    @Log(title = "文件审批业务", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(fileApprovalService.removeByIds(Arrays.asList(ids)));
    }
}
