package com.willalp.web.controller.clockingin;

import com.willalp.common.annotation.Log;
import com.willalp.common.core.controller.BaseController;
import com.willalp.common.core.domain.AjaxResult;
import com.willalp.common.core.page.TableDataInfo;
import com.willalp.common.enums.BusinessType;
import com.willalp.common.utils.DateUtils;
import com.willalp.common.utils.SecurityUtils;
import com.willalp.common.utils.poi.ExcelUtil;
import com.willalp.flow.domain.HsContractManage;
import com.willalp.flow.service.IHsContractManageService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;

/**
 * 合同管理记录Controller
 *
 * @author willalp
 * @date 2023-05-24
 */
@RestController
@RequestMapping("/flow/contract")
public class HsContractManageController extends BaseController
{
    @Resource
    private IHsContractManageService hsContractManageService;

    /**
     * 查询合同管理记录列表
     */
    @PreAuthorize("@ss.hasPermi('flow:contract:list')")
    @GetMapping("/list")
    public TableDataInfo list(HsContractManage hsContractManage)
    {
        startPage();
        List<HsContractManage> list = hsContractManageService.selectHsContractManageList(hsContractManage);
        return getDataTable(list);
    }

    /**
     * 导出合同管理记录列表
     */
    @PreAuthorize("@ss.hasPermi('flow:contract:export')")
    @Log(title = "合同管理记录", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(HsContractManage hsContractManage)
    {
        List<HsContractManage> list = hsContractManageService.selectHsContractManageList(hsContractManage);
        ExcelUtil<HsContractManage> util = new ExcelUtil<HsContractManage>(HsContractManage.class);
        return util.exportExcel(list, "合同管理记录数据");
    }

    /**
     * 获取合同管理记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('flow:contract:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id)
    {
        return AjaxResult.success(hsContractManageService.getById(id));
    }

    /**
     * 新增合同管理记录
     */
    @PreAuthorize("@ss.hasPermi('flow:contract:add')")
    @Log(title = "合同管理记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody HsContractManage hsContractManage)
    {
        hsContractManage.setCreateBy(SecurityUtils.getUsername());
        hsContractManage.setCreateTime(DateUtils.getNowDate());
        return toAjax(hsContractManageService.save(hsContractManage));
    }

    /**
     * 修改合同管理记录
     */
    @PreAuthorize("@ss.hasPermi('flow:contract:edit')")
    @Log(title = "合同管理记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody HsContractManage hsContractManage)
    {
        hsContractManage.setUpdateBy(SecurityUtils.getUsername());
        hsContractManage.setUpdateTime(DateUtils.getNowDate());
        return toAjax(hsContractManageService.updateById(hsContractManage));
    }

    /**
     * 删除合同管理记录
     */
    @PreAuthorize("@ss.hasPermi('flow:contract:remove')")
    @Log(title = "合同管理记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids)
    {
        return toAjax(hsContractManageService.removeByIds(Arrays.asList(ids)));
    }
}
