package com.willalp.web.controller.clockingin;

import com.willalp.common.annotation.Log;
import com.willalp.common.core.controller.BaseController;
import com.willalp.common.core.domain.AjaxResult;
import com.willalp.common.core.page.TableDataInfo;
import com.willalp.common.enums.BusinessType;
import com.willalp.common.utils.DateUtils;
import com.willalp.common.utils.SecurityUtils;
import com.willalp.common.utils.poi.ExcelUtil;
import com.willalp.flow.domain.HsLetterDispose;
import com.willalp.flow.service.IHsLetterDisposeService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;

/**
 * 信访件处理记录Controller
 *
 * @author willalp
 * @date 2023-06-27
 */
@RestController
@RequestMapping("/flow/dispose")
public class HsLetterDisposeController extends BaseController
{
    @Resource
    private IHsLetterDisposeService hsLetterDisposeService;

    /**
     * 查询信访件处理记录列表
     */
    @PreAuthorize("@ss.hasPermi('flow:dispose:list')")
    @GetMapping("/list")
    public TableDataInfo list(HsLetterDispose hsLetterDispose)
    {
        startPage();
        List<HsLetterDispose> list = hsLetterDisposeService.selectHsLetterDisposeList(hsLetterDispose);
        return getDataTable(list);
    }

    /**
     * 导出信访件处理记录列表
     */
    @PreAuthorize("@ss.hasPermi('flow:dispose:export')")
    @Log(title = "信访件处理记录", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(HsLetterDispose hsLetterDispose)
    {
        List<HsLetterDispose> list = hsLetterDisposeService.selectHsLetterDisposeList(hsLetterDispose);
        ExcelUtil<HsLetterDispose> util = new ExcelUtil<HsLetterDispose>(HsLetterDispose.class);
        return util.exportExcel(list, "信访件处理记录数据");
    }

    /**
     * 获取信访件处理记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('flow:dispose:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id)
    {
        return AjaxResult.success(hsLetterDisposeService.getById(id));
    }

    /**
     * 新增信访件处理记录
     */
    @PreAuthorize("@ss.hasPermi('flow:dispose:add')")
    @Log(title = "信访件处理记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody HsLetterDispose hsLetterDispose)
    {
        hsLetterDispose.setCreateBy(SecurityUtils.getUsername());
        hsLetterDispose.setCreateTime(DateUtils.getNowDate());
        return toAjax(hsLetterDisposeService.save(hsLetterDispose));
    }

    /**
     * 修改信访件处理记录
     */
    @PreAuthorize("@ss.hasPermi('flow:dispose:edit')")
    @Log(title = "信访件处理记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody HsLetterDispose hsLetterDispose)
    {
        hsLetterDispose.setUpdateBy(SecurityUtils.getUsername());
        hsLetterDispose.setUpdateTime(DateUtils.getNowDate());
        return toAjax(hsLetterDisposeService.updateById(hsLetterDispose));
    }

    /**
     * 删除信访件处理记录
     */
    @PreAuthorize("@ss.hasPermi('flow:dispose:remove')")
    @Log(title = "信访件处理记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids)
    {
        return toAjax(hsLetterDisposeService.removeByIds(Arrays.asList(ids)));
    }
}
