package com.willalp.web.controller.clockingin;

import com.willalp.common.annotation.Log;
import com.willalp.common.core.controller.BaseController;
import com.willalp.common.core.domain.AjaxResult;
import com.willalp.common.core.page.TableDataInfo;
import com.willalp.common.enums.BusinessType;
import com.willalp.common.utils.DateUtils;
import com.willalp.common.utils.SecurityUtils;
import com.willalp.common.utils.poi.ExcelUtil;
import com.willalp.flow.domain.HsPhoneApply;
import com.willalp.flow.service.IHsPhoneApplyService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;

/**
 * 手机号申请记录Controller
 *
 * @author willalp
 * @date 2023-06-27
 */
@RestController
@RequestMapping("/flow/phone-apply")
public class HsPhoneApplyController extends BaseController
{
    @Resource
    private IHsPhoneApplyService hsPhoneApplyService;

    /**
     * 查询手机号申请记录列表
     */
    @PreAuthorize("@ss.hasPermi('flow:phone-apply:list')")
    @GetMapping("/list")
    public TableDataInfo list(HsPhoneApply hsPhoneApply)
    {
        startPage();
        List<HsPhoneApply> list = hsPhoneApplyService.selectHsPhoneApplyList(hsPhoneApply);
        return getDataTable(list);
    }

    /**
     * 导出手机号申请记录列表
     */
    @PreAuthorize("@ss.hasPermi('flow:phone-apply:export')")
    @Log(title = "手机号申请记录", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(HsPhoneApply hsPhoneApply)
    {
        List<HsPhoneApply> list = hsPhoneApplyService.exportData(hsPhoneApply);
        ExcelUtil<HsPhoneApply> util = new ExcelUtil<HsPhoneApply>(HsPhoneApply.class);
        return util.exportExcel(list, "手机号申请记录数据");
    }

    /**
     * 获取手机号申请记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('flow:phone-apply:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id)
    {
        return AjaxResult.success(hsPhoneApplyService.getById(id));
    }

    /**
     * 新增手机号申请记录
     */
    @PreAuthorize("@ss.hasPermi('flow:phone-apply:add')")
    @Log(title = "手机号申请记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody HsPhoneApply hsPhoneApply)
    {
        hsPhoneApply.setCreateBy(SecurityUtils.getUsername());
        hsPhoneApply.setCreateTime(DateUtils.getNowDate());
        return toAjax(hsPhoneApplyService.save(hsPhoneApply));
    }

    /**
     * 修改手机号申请记录
     */
    @PreAuthorize("@ss.hasPermi('flow:phone-apply:edit')")
    @Log(title = "手机号申请记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody HsPhoneApply hsPhoneApply)
    {
        hsPhoneApply.setUpdateBy(SecurityUtils.getUsername());
        hsPhoneApply.setUpdateTime(DateUtils.getNowDate());
        return toAjax(hsPhoneApplyService.updateById(hsPhoneApply));
    }

    /**
     * 删除手机号申请记录
     */
    @PreAuthorize("@ss.hasPermi('flow:phone-apply:remove')")
    @Log(title = "手机号申请记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids)
    {
        return toAjax(hsPhoneApplyService.removeByIds(Arrays.asList(ids)));
    }
}
