package com.willalp.web.controller.clockingin;

import java.util.List;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.willalp.common.annotation.Log;
import com.willalp.common.core.controller.BaseController;
import com.willalp.common.core.domain.AjaxResult;
import com.willalp.common.enums.BusinessType;
import com.willalp.common.utils.DateUtils;
import com.willalp.common.utils.SecurityUtils;
import javax.annotation.Resource;
import java.util.Arrays;
import com.willalp.flow.domain.HsUseSeal;
import com.willalp.flow.service.IHsUseSealService;
import com.willalp.common.utils.poi.ExcelUtil;
import com.willalp.common.core.page.TableDataInfo;

/**
 * 用章申请流程记录Controller
 *
 * @author willalp
 * @date 2023-05-26
 */
@RestController
@RequestMapping("/flow/useSeal")
public class HsUseSealController extends BaseController
{
    @Resource
    private IHsUseSealService hsUseSealService;

    /**
     * 查询用章申请流程记录列表
     */
    @PreAuthorize("@ss.hasPermi('flow:useSeal:list')")
    @GetMapping("/list")
    public TableDataInfo list(HsUseSeal hsUseSeal)
    {
        startPage();
        List<HsUseSeal> list = hsUseSealService.selectHsUseSealList(hsUseSeal);
        return getDataTable(list);
    }

    /**
     * 导出用章申请流程记录列表
     */
    @PreAuthorize("@ss.hasPermi('flow:useSeal:export')")
    @Log(title = "用章申请流程记录", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(HsUseSeal hsUseSeal)
    {
        List<HsUseSeal> list = hsUseSealService.exportHsUseSealList(hsUseSeal);
        ExcelUtil<HsUseSeal> util = new ExcelUtil<HsUseSeal>(HsUseSeal.class);
        return util.exportExcel(list, "用章申请流程记录数据");
    }

    /**
     * 获取用章申请流程记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('flow:useSeal:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id)
    {
        return AjaxResult.success(hsUseSealService.getById(id));
    }

    /**
     * 新增用章申请流程记录
     */
    @PreAuthorize("@ss.hasPermi('flow:useSeal:add')")
    @Log(title = "用章申请流程记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody HsUseSeal hsUseSeal)
    {
        hsUseSeal.setCreateBy(SecurityUtils.getUsername());
        hsUseSeal.setCreateTime(DateUtils.getNowDate());
        return toAjax(hsUseSealService.save(hsUseSeal));
    }

    /**
     * 修改用章申请流程记录
     */
    @PreAuthorize("@ss.hasPermi('flow:useSeal:edit')")
    @Log(title = "用章申请流程记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody HsUseSeal hsUseSeal)
    {
        hsUseSeal.setUpdateBy(SecurityUtils.getUsername());
        hsUseSeal.setUpdateTime(DateUtils.getNowDate());
        return toAjax(hsUseSealService.updateById(hsUseSeal));
    }

    /**
     * 删除用章申请流程记录
     */
    @PreAuthorize("@ss.hasPermi('flow:useSeal:remove')")
    @Log(title = "用章申请流程记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids)
    {
        return toAjax(hsUseSealService.removeByIds(Arrays.asList(ids)));
    }
}
