package com.willalp.event.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.willalp.common.core.domain.ReturnEntity;
import com.willalp.common.core.domain.entity.SysDictData;
import com.willalp.common.core.domain.entity.SysUser;
import com.willalp.common.exception.base.BaseException;
import com.willalp.common.utils.DateUtils;
import com.willalp.common.utils.ObjectToTypes;
import com.willalp.common.utils.StringUtils;
import com.willalp.event.domain.HsWelfareApplyRecord;
import com.willalp.event.domain.vo.WelfareApplyStatVo;
import com.willalp.event.mapper.HsWelfareApplyRecordMapper;
import com.willalp.event.service.IHsWelfareApplyRecordService;
import com.willalp.msg.domain.HsAppUserMsg;
import com.willalp.msg.service.IHsAppUserMsgService;
import com.willalp.push.service.IHsAppPushService;
import com.willalp.system.service.ISysDictTypeService;
import com.willalp.wx.domain.WxMessageBeanApprovalGeneral;
import com.willalp.wx.domain.WxMessageBeanApprovalWait;
import com.willalp.wx.enums.WxTemplateKeyEnum;
import com.willalp.wx.service.WxMsgService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 福利报名记录Service业务层处理
 *
 * @author willalp
 * @date 2023-04-17
 */
@Service
public class HsWelfareApplyRecordServiceImpl extends ServiceImpl<HsWelfareApplyRecordMapper, HsWelfareApplyRecord> implements IHsWelfareApplyRecordService {
    @Resource
    private HsWelfareApplyRecordMapper hsWelfareApplyRecordMapper;

    @Resource
    private ISysDictTypeService dictTypeService;

    /**
     * 查询福利报名记录列表
     *
     * @param hsWelfareApplyRecord 福利报名记录
     * @return 福利报名记录
     */
    @Override
    public List<HsWelfareApplyRecord> selectHsWelfareApplyRecordList(HsWelfareApplyRecord hsWelfareApplyRecord) {
        return hsWelfareApplyRecordMapper.selectHsWelfareApplyRecordList(hsWelfareApplyRecord);
    }

    @Override
    public List<HsWelfareApplyRecord> importInitDataTemp() {
        return hsWelfareApplyRecordMapper.importInitDataTemp();
    }

    @Override
    public List<HsWelfareApplyRecord> exportDataAndInterpret(HsWelfareApplyRecord hsWelfareApplyRecord) {
        List<HsWelfareApplyRecord> list = hsWelfareApplyRecordMapper.selectHsWelfareApplyRecordList(hsWelfareApplyRecord);
        List<SysDictData> applyAddr = dictTypeService.selectDictDataByType("welfare_apply_addr");
        List<SysDictData> applyStatus = dictTypeService.selectDictDataByType("welfare_apply_status");
        list.forEach(item -> {
            applyAddr.forEach(dict -> {
                if (StringUtils.isNotBlank(item.getAddress()) && item.getAddress().equals(dict.getDictValue())) {
                    item.setAddress(dict.getDictLabel());
                }
            });
            applyStatus.forEach(dict -> {
                if (StringUtils.isNotBlank(item.getNowStatus()) && item.getNowStatus().equals(dict.getDictValue())) {
                    item.setNowStatus(dict.getDictLabel());
                }
            });
        });
        return list;
    }

    @Override
    public String importData(List<HsWelfareApplyRecord> list, String fileName) {
        if (StringUtils.isBlank(fileName)) {
            if (count(new QueryWrapper<HsWelfareApplyRecord>().eq("source", fileName)) > 0) {
                throw new BaseException("该Excel数据已经存在，请检查或重命名");
            }
        }
        int successNum = 0;
        int failureNum = 0;
        for (HsWelfareApplyRecord welfareApplyRecord : list) {
            try {
                welfareApplyRecord.setSource(fileName);
                welfareApplyRecord.setNowStatus("1");
                save(welfareApplyRecord);
                successNum++;
            } catch (Exception e) {
                failureNum++;
                log.error(e.getMessage(), e);
            }
        }
        ReturnEntity<String> res = new ReturnEntity<>();
        res.setReturnMessage("event.club.successCode", new String[]{
                ObjectToTypes.parseString(successNum), ObjectToTypes.parseString(failureNum)
        });
        return res.getMessage();
    }

    @Override
    public List<WelfareApplyStatVo> stat(String source) {
        if (StringUtils.isBlank(source)) {
            return new ArrayList<>();
        }
        return baseMapper.stat(source);
    }

    @Resource
    private IHsAppUserMsgService userMsgService;

    @Resource
    private WxMsgService wxMsgService;

    @Resource
    private IHsAppPushService pushService;

    @Override
    public int doUnregistered(String source) {
        if (StringUtils.isBlank(source)) {
            return 0;
        }
        HsWelfareApplyRecord welfareApplyRecord = new HsWelfareApplyRecord();
        welfareApplyRecord.setSource(source);
        List<HsWelfareApplyRecord> list = baseMapper.selectHsWelfareApplyRecordList(welfareApplyRecord);
        if (list.size() > 0) {
            if (list.get(0).getEndTime() == null) {
                throw new BaseException("请先设置截止时间");
            }
            Date createTime = new Date();
            WxMessageBeanApprovalGeneral general = new WxMessageBeanApprovalGeneral(source, "待处理",
                    DateUtils.dateTime(createTime) + " - 至 - " + DateUtils.dateTime(list.get(0).getEndTime()));
            List<HsAppUserMsg> msgList = new ArrayList<>(32);
            if (list.size() > 0) {
                for (HsWelfareApplyRecord record : list) {
                    HsAppUserMsg appUserMsg = new HsAppUserMsg();
                    appUserMsg.setMsgTitle("活动预约报名通知");
                    appUserMsg.setUserCode(record.getUserCode());
                    appUserMsg.setMsgIsRead("0");
                    appUserMsg.setMsgType("8");
                    appUserMsg.setMsgContent("您的【" + source + "】待处理");
                    appUserMsg.setCreateTime(createTime);
                    msgList.add(appUserMsg);
                }
                if (userMsgService.saveBatch(msgList)) {
                    //微信推送
                    for (HsWelfareApplyRecord record : list) {
                        if (StringUtils.isBlank(record.getUserCode())) {
                            break;
                        }
                        String openIdByUserNo = pushService.getOpenIdByUserNo(record.getUserCode());
                        if (StringUtils.isNotBlank(openIdByUserNo)) {
                            wxMsgService.sendQRCodeMsg(openIdByUserNo, general, WxTemplateKeyEnum.GENERAL_BEFORE_WARN);
                        }
                    }
                }
            }
        } else {
            throw new BaseException("没有该福利报名数据，请导入");
        }
        return baseMapper.doUnregistered(source);
    }


}
