package com.willalp.files.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.willalp.common.core.domain.entity.SysDept;
import com.willalp.common.core.domain.entity.SysDictData;
import com.willalp.common.exception.base.BaseException;
import com.willalp.common.utils.StringUtils;
import com.willalp.files.domain.HsOrgPersonnelFiles;
import com.willalp.files.mapper.HsOrgPersonnelFilesMapper;
import com.willalp.files.service.IHsOrgPersonnelFilesService;
import com.willalp.system.domain.SysDeptPost;
import com.willalp.system.domain.SysUserOrganization;
import com.willalp.system.service.*;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 机构人员档案Service业务层处理
 *
 * @author willalp
 * @date 2022-08-01
 */
@Service
public class HsOrgPersonnelFilesServiceImpl extends ServiceImpl<HsOrgPersonnelFilesMapper, HsOrgPersonnelFiles> implements IHsOrgPersonnelFilesService {
    @Resource
    private HsOrgPersonnelFilesMapper hsOrgPersonnelFilesMapper;

    @Resource
    private ISysDictTypeService dictTypeService;

    @Resource
    private ISysDeptService deptService;

    @Resource
    private ISysUserOrganizationService sysUserOrganizationService;

    /**
     * 查询机构人员档案列表
     *
     * @param hsOrgPersonnelFiles 机构人员档案
     * @return 机构人员档案
     */
    @Override
    public List<HsOrgPersonnelFiles> selectHsOrgPersonnelFilesList(HsOrgPersonnelFiles hsOrgPersonnelFiles) {
        return hsOrgPersonnelFilesMapper.selectHsOrgPersonnelFilesList(hsOrgPersonnelFiles);
    }

    @Override
    public List<HsOrgPersonnelFiles> importHsOrgPersonnelFilesList(HsOrgPersonnelFiles hsOrgPersonnelFiles) {
        List<HsOrgPersonnelFiles> dbList = hsOrgPersonnelFilesMapper.selectHsOrgPersonnelFilesList(hsOrgPersonnelFiles);
        List<SysDictData> jobTitleData = dictTypeService.selectDictDataByType("job_title");
        List<SysDictData> userType = dictTypeService.selectDictDataByType("user_type");
        List<SysDeptPost> sysDeptPosts = sysDeptPostService.selectSysDeptPostList(null);
        List<SysDictData> personnelNowStatus = dictTypeService.selectDictDataByType("personnel_now_status");
        dbList.forEach(dbItem -> {
            for (SysDictData item : jobTitleData) {
                if (StringUtils.isNotBlank(dbItem.getJobTitle())
                        && dbItem.getJobTitle().equals(item.getDictValue())) {
                    dbItem.setJobTitle(item.getDictLabel());
                    break;
                }
            }
            for (SysDictData item : userType) {
                if (StringUtils.isNotBlank(dbItem.getUserType())
                        && dbItem.getUserType().equals(item.getDictValue())) {
                    dbItem.setUserType(item.getDictLabel());
                    break;
                }
            }
            for (SysDeptPost item : sysDeptPosts) {
                if (StringUtils.isNotBlank(dbItem.getPost())
                        && dbItem.getPost().equals(item.getPostId())) {
                    dbItem.setPost(item.getPostName());
                    break;
                }
            }
            for (SysDictData item : personnelNowStatus) {
                if (StringUtils.isNotBlank(dbItem.getNowStatus())
                        && dbItem.getNowStatus().equals(item.getDictValue())) {
                    dbItem.setNowStatus(item.getDictLabel());
                    break;
                }
            }
        });
        return dbList;
    }

    @Override
    public String importData(List<HsOrgPersonnelFiles> list) {
        List<HsOrgPersonnelFiles> dbList = this.selectHsOrgPersonnelFilesList(null);
        SysUserOrganization sysUserOrganization = new SysUserOrganization();
        sysUserOrganization.setOrganizationCode("G00000007");
        List<SysUserOrganization> sysUserOrganizations = sysUserOrganizationService.selectSysUserOrganizationList(sysUserOrganization);

        List<SysDictData> jobTitleData = dictTypeService.selectDictDataByType("job_title");

        List<SysDictData> personnelNowStatus = dictTypeService.selectDictDataByType("personnel_now_status");

        List<SysDictData> userTypes = dictTypeService.selectDictDataByType("user_type");

        Map<String, Map<String, Object>> deptMap = deptService.selectDeptByMapKeyIsName();

        Map<String, Map<String, Object>> mapPostKeyIsName = sysDeptPostService.getMapPostKeyIsName();
        List<HsOrgPersonnelFiles> savaList = new ArrayList<>();
        List<HsOrgPersonnelFiles> updateList = new ArrayList<>();
        List<SysUserOrganization> sous = new ArrayList<>();

        for (HsOrgPersonnelFiles hsOrgPersonnelFiles : list) {
            if (StringUtils.isBlank(hsOrgPersonnelFiles.getUserCode())) {
                break;
            }
            //字典值写入
            String jobTitle = hsOrgPersonnelFiles.getJobTitle();
            hsOrgPersonnelFiles.setJobTitle(importTranslation(jobTitleData, jobTitle));
            String nowStatus = hsOrgPersonnelFiles.getNowStatus();
            hsOrgPersonnelFiles.setNowStatus(importTranslation(personnelNowStatus, nowStatus));

            String post = hsOrgPersonnelFiles.getPost();
            if (StringUtils.isNotBlank(post)) {
                if (null != mapPostKeyIsName.get(post) && null != mapPostKeyIsName.get(post).get("post_id")) {
                    hsOrgPersonnelFiles.setPost(String.valueOf(mapPostKeyIsName.get(post).get("post_id")));
                }
            }
            String department = hsOrgPersonnelFiles.getDepartment();
            if (StringUtils.isNotBlank(department)) {
                if (null != deptMap) {
                    Map<String, Object> map = deptMap.get(department);
                    if (null != map && null != map.get("dept_name")) {
                        hsOrgPersonnelFiles.setDepartment(String.valueOf(map.get("dept_id")));
                    }
                }
            }
            int count = 0;
            for (int i = 0; i < dbList.size(); i++) {
                if (hsOrgPersonnelFiles.getUserCode().equals(dbList.get(i).getUserCode())) {
                    count++;
                    hsOrgPersonnelFiles.setId(dbList.get(i).getId());
                    String userType = hsOrgPersonnelFiles.getUserType();
                    String newUserType = importTranslation(userTypes, userType);
                    if (StringUtils.isNotBlank(newUserType) && !newUserType.equals(dbList.get(i).getUserType())) {
                        for (SysUserOrganization sou : sysUserOrganizations) {
                            if (sou.getLoginName().equals(hsOrgPersonnelFiles.getUserCode())) {
                                sou.setUserType(newUserType);
                                sous.add(sou);
                                break;
                            }
                        }
                    }
                    break;
                }
            }
            if (count > 0) {
                updateList.add(hsOrgPersonnelFiles);
            } else {
                savaList.add(hsOrgPersonnelFiles);
            }
        }

        if (0 != list.size()) {
            if (0 != savaList.size()) {
                if (!saveBatch(savaList)) {
                    throw new BaseException("数据导入失败");
                }
            }
            if (0 != updateList.size()) {
                if (!updateBatchById(updateList)) {
                    throw new BaseException("数据更新失败");
                }
            }
            if (0 != sous.size()) {
                if (!sysUserOrganizationService.updateBatchById(sous)) {
                    throw new BaseException("数据导入失败");
                }
            }
        }
        return "共导入【" + savaList.size() + "】条新数据，更新【" + updateList.size() + "】条数据";
    }

    private String importTranslation(List<SysDictData> list, String label) {
        if (StringUtils.isNotBlank(label)) {
            if (0 != list.size()) {
                List<SysDictData> collect = list.stream()
                        .filter(item -> StringUtils.isNotBlank(item.getDictLabel()) && item.getDictLabel().equals(label))
                        .collect(Collectors.toList());
                if (0 != collect.size()) {
                    return collect.get(0).getDictValue();
                }
            }
        }
        return "";
    }


    @Resource
    private ISysDeptService sysDeptService;

    @Resource
    private ISysDeptPostService sysDeptPostService;

    @Override
    public HsOrgPersonnelFiles appGetHsOrgPersonnelFilesByPhone(String phone) {
        if (StringUtils.isBlank(phone)) {
            throw new BaseException("手机号为空，请添加档案信息");
        }
        HsOrgPersonnelFiles orgPersonnelFiles = this.getBaseMapper().selectOne(new QueryWrapper<HsOrgPersonnelFiles>()
                .eq("phone", phone));
        if (orgPersonnelFiles == null) {
            throw new BaseException("暂未查询到该用户名片信息");
        }
        //补充部门信息
        String department = orgPersonnelFiles.getDepartment();
        if (StringUtils.isNotBlank(department)) {
            SysDept sysDept = sysDeptService.selectDeptById(Long.valueOf(department));
            orgPersonnelFiles.setDepartment(sysDept.getDeptName());
        } else {
            orgPersonnelFiles.setDepartment("暂无");
        }
        //补充岗位信息
        String post = orgPersonnelFiles.getPost();
        if (StringUtils.isNotBlank(post)) {
            SysDeptPost sysDeptPost = sysDeptPostService.selectById(Long.valueOf(post));
            orgPersonnelFiles.setPost(sysDeptPost.getPostName());
        } else {
            orgPersonnelFiles.setPost("暂无");
        }
        String seatNo = orgPersonnelFiles.getSeatNo();
        if (StringUtils.isBlank(seatNo)) {
            orgPersonnelFiles.setSeatNo("暂无");
        }
        String email = orgPersonnelFiles.getEmail();
        if (StringUtils.isBlank(email)) {
            orgPersonnelFiles.setEmail("暂无");
        }
        return orgPersonnelFiles;
    }

    @Override
    public Map<String, Object> flowGetFilesById(String id) {
        return baseMapper.flowGetFilesById(id);
    }
}
