/**
 * @projectName WILLALP-PNS-ML
 * @package com.willalp.flow.service
 * @className com.willalp.flow.service.impl.FlowTriggerServiceImpl
 * @copyright Copyright 2019 HuiShan, Inc All rights reserved.
 */
package com.willalp.flow.service.impl;

import cn.hutool.json.JSONObject;
import com.willalp.common.exception.base.BaseException;
import com.willalp.common.utils.DateUtils;
import com.willalp.common.utils.StringUtils;
import com.willalp.erms.domain.HsGoodsStorage;
import com.willalp.erms.service.IHsGoodsStorageService;
import com.willalp.files.domain.HsOrgPersonnelFiles;
import com.willalp.files.service.IHsOrgPersonnelFilesService;
import com.willalp.flow.domain.HsFlowTask;
import com.willalp.flow.service.FlowTriggerService;
import com.willalp.flow.service.HsFlowEngineService;
import com.willalp.flow.service.IHsFlowTaskService;
import com.willalp.push.service.IHsAppPushService;
import com.willalp.wx.domain.WxMessageBeanApprovalGeneral;
import com.willalp.wx.enums.WxTemplateKeyEnum;
import com.willalp.wx.service.WxMsgService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * FlowTriggerServiceImpl
 * @description
 * @author hs
 * @date 2023/5/24 16:06
 * @version 1.0
 */
@Service
public class FlowTriggerServiceImpl implements FlowTriggerService {

    protected final static Logger logger = LoggerFactory.getLogger(FlowTriggerServiceImpl.class);

    @Resource
    private HsFlowEngineService flowEngineService;

    @Resource
    private IHsFlowTaskService flowTaskService;

    @Resource
    private IHsAppPushService pushService;

    @Resource
    private WxMsgService wxMsgService;


    /**
     *
     * @Title: doMethod
     * @Description:
     * @param methodName 函数名称
     * @param methodParam 参数集合 List
     * @return
     * @throws Exception Object
     */
    @Override
    public Object doMethod(String methodName, List<Map<String, Object>> methodParam) throws Exception {
        int methodParamSize = methodParam.size();
        Class[] methodParamType = new Class[]{};
        Object[] methodParamValue = new Object[]{};
        if (methodParamSize > 0) {
            methodParamType = new Class[methodParamSize];
            methodParamValue = new Object[methodParamSize];

            for (int i = 0; i < methodParamSize; i++) {
                methodParamType[i] = (Class) methodParam.get(i).get("argType");
                methodParamValue[i] = methodParam.get(i).get("argValue");
            }
        }
        return this.getClass().getMethod(methodName, methodParamType).invoke(this, methodParamValue);
    }

    @Override
    public JSONObject getTaskBizData(String taskId) {
        if (StringUtils.isBlank(taskId)) {
            throw new BaseException("触发器异常 - taskID为空");
        }
        HsFlowTask flowTask = flowTaskService.getById(taskId);
        List<HsFlowTask> list = new ArrayList<>();
        list.add(flowTask);
        List<HsFlowTask> hsFlowTasks = flowEngineService.getHsFlowTasks(list);
        HsFlowTask task = hsFlowTasks.get(0);
        JSONObject formData = task.getFormData();
        if (formData == null) {
            throw new BaseException("触发器异常 - task_biz_data_is_null");
        }
        return formData;
    }


    @Resource
    private IHsGoodsStorageService goodsStorageService;


    @Override
    public void goodsStorageUpdate(String taskId, String param) {
        if (StringUtils.isBlank(param)) {
            throw new BaseException("物资库存数量更新失败 - 更新类型为空");
        }
        //获取流程任务业务数据
        JSONObject formData = getTaskBizData(taskId);

        String goodsId = String.valueOf(formData.get("goods_id"));
        int amount = (Integer) formData.get("amount");
        String recordType = (String) formData.get("record_type");

        if (StringUtils.isBlank(goodsId)) {
            throw new BaseException("物资库存数量更新失败 - 物资ID为空");
        }
        if (StringUtils.isBlank(recordType)) {
            throw new BaseException("物资库存数量更新失败 - 更新类型为空");
        }
        HsGoodsStorage goodsItemOld = goodsStorageService.getById(goodsId);
        if (goodsItemOld == null) {
            throw new BaseException("物资库存数量更新失败 - 该物资不存在");
        }
        if (goodsStorageService.updateQuantityInStock(goodsId, recordType, amount) > 0) {
            HsGoodsStorage goodsItemNew = goodsStorageService.getById(goodsId);
            String goodsName;
            if (StringUtils.isNotBlank(goodsItemOld.getName())) {
                goodsName = goodsItemOld.getName();
            } else {
                goodsName = goodsId;
            }
            logger.info("物资库存数量更新成功 - "
                    + goodsName + "【"
                    + goodsItemOld.getQuantityInStock()
                    + " -> "
                    + goodsItemNew.getQuantityInStock()
                    + "】");
        }
    }

    @Resource
    private IHsOrgPersonnelFilesService opfService;

    @Override
    public void orgPersonnelFilesUpdate(String taskId) {
        //获取流程任务业务数据
        JSONObject formData = getTaskBizData(taskId);
        String personnelId = String.valueOf(formData.get("personnel_id"));
        if (StringUtils.isBlank(personnelId)) {
            throw new BaseException("触发器 - 员工档案 - 更新失败 - 参数为空");
        }
        String hireOrDimission = (String) formData.get("hire_or_dimission");
        if (StringUtils.isBlank(hireOrDimission)) {
            throw new BaseException("触发器 - 员工档案 - 更新失败 - 参数为空");
        }
        HsOrgPersonnelFiles opf = opfService.getById(personnelId);
        if (opf == null) {
            throw new BaseException("触发器 - 更新失败 - 员工档案为空");
        }
        String statusText = "";
        if ("1".equals(hireOrDimission)) {
            opf.setNowStatus("2");
            statusText = "入职";
        } else if ("2".equals(hireOrDimission)) {
            opf.setNowStatus("5");
            opf.setResignationTime(DateUtils.parseDate(formData.get("apply_time")));
            statusText = "离职";
        }
        if (opfService.updateById(opf)) {
            logger.info("触发器 - update - 员工【" + opf.getUserName() + "】-> 【" + statusText + "】");
        }
    }

    @Override
    public void phoneApplyEndPush(String taskId) {
        //获取流程任务业务数据
        JSONObject formData = getTaskBizData(taskId);
        if (formData != null) {
            String phoneNum = String.valueOf(formData.get("phone_num"));
            String applyUserId = String.valueOf(formData.get("apply_user_id"));
            WxMessageBeanApprovalGeneral general = new WxMessageBeanApprovalGeneral("您的手机卡申请已通过", "待领取",
                    "卡号为：【" + phoneNum + "】请前往行政办公室领用");
            String openid = pushService.getOpenIdByUserNo(applyUserId);
            wxMsgService.sendQRCodeMsg(openid, general, WxTemplateKeyEnum.GENERAL_BEFORE_WARN);
        }
    }

}
 