package com.willalp.flow.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.willalp.common.core.domain.entity.SysDictData;
import com.willalp.common.utils.DateUtils;
import com.willalp.common.utils.StringUtils;
import com.willalp.flow.domain.HsUseSeal;
import com.willalp.flow.mapper.HsUseSealMapper;
import com.willalp.flow.service.IHsUseSealService;
import com.willalp.system.service.ISysDictTypeService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 用章申请流程记录Service业务层处理
 *
 * @author willalp
 * @date 2023-03-31
 */
@Service
public class HsUseSealServiceImpl extends ServiceImpl<HsUseSealMapper, HsUseSeal> implements IHsUseSealService {

    @Resource
    private ISysDictTypeService dictTypeService;

    /**
     * 查询用章申请流程记录列表
     *
     *
     * @param hsUseSeal 用章申请流程记录
     * @return 用章申请流程记录
     */
    @Override
    public List<HsUseSeal> selectHsUseSealList(HsUseSeal hsUseSeal) {
        return baseMapper.selectHsUseSealList(hsUseSeal);
    }

    @Override
    public List<HsUseSeal> exportHsUseSealList(HsUseSeal hsUseSeal) {
        List<HsUseSeal> hsUseSeals = baseMapper.selectHsUseSealList(hsUseSeal);
        if (hsUseSeals.size() == 0) {
            return null;
        }

        List<SysDictData> seal_type = dictTypeService.selectDictDataByType("seal_type");
        List<SysDictData> hzl_seal_use_type = dictTypeService.selectDictDataByType("hzl_seal_use_type");
        List<SysDictData> hzl_seal_use_address = dictTypeService.selectDictDataByType("hzl_seal_use_address");
        List<SysDictData> hzl_file_type = dictTypeService.selectDictDataByType("hzl_file_type");
        for (HsUseSeal useSeal : hsUseSeals) {
            useSeal.setUsA(importTranslation(hzl_seal_use_type, useSeal.getUsA()));
            useSeal.setUsB(importTranslation(seal_type, useSeal.getUsB()));
            useSeal.setUsE(importTranslation(hzl_seal_use_address, useSeal.getUsE()));
            useSeal.setSoG(importTranslation(hzl_file_type, useSeal.getSoG()));
            if (null != useSeal.getCreateTime()) {
                useSeal.setCreateDateTime(DateUtils.dateTime(useSeal.getCreateTime()));
            }
        }
        return hsUseSeals;
    }

    private String importTranslation(List<SysDictData> list, String value) {
        if (StringUtils.isNotBlank(value)) {
            if (0 != list.size()) {
                List<SysDictData> collect = list.stream()
                        .filter(item -> StringUtils.isNotBlank(item.getDictValue()) && item.getDictValue().equals(value))
                        .collect(Collectors.toList());
                if (0 != collect.size()) {
                    return collect.get(0).getDictLabel();
                }
            }
        }
        return "";
    }

}
