package com.willalp.erms.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.willalp.common.core.domain.entity.SysDept;
import com.willalp.common.core.domain.entity.SysDictData;
import com.willalp.common.utils.StringUtils;
import com.willalp.erms.domain.HsGoodsStorageRecord;
import com.willalp.erms.domain.vo.DeptStatListVo;
import com.willalp.erms.domain.vo.DeptStatVo;
import com.willalp.erms.mapper.HsGoodsStorageRecordMapper;
import com.willalp.erms.service.IHsGoodsStorageRecordService;
import com.willalp.system.service.ISysDeptService;
import com.willalp.system.service.ISysDictTypeService;
import org.apache.ibatis.cursor.Cursor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 物资库存记录Service业务层处理
 *
 * @author willalp
 * @date 2023-03-08
 */
@Service
public class HsGoodsStorageRecordServiceImpl extends ServiceImpl<HsGoodsStorageRecordMapper, HsGoodsStorageRecord> implements IHsGoodsStorageRecordService {
    @Resource
    private HsGoodsStorageRecordMapper hsGoodsStorageRecordMapper;

    /**
     * 查询物资库存记录列表
     *
     * @param hsGoodsStorageRecord 物资库存记录
     * @return 物资库存记录
     */
    @Override
    public List<HsGoodsStorageRecord> selectHsGoodsStorageRecordList(HsGoodsStorageRecord hsGoodsStorageRecord) {
        return hsGoodsStorageRecordMapper.selectHsGoodsStorageRecordList(hsGoodsStorageRecord);
    }

    @Resource
    private ISysDictTypeService dictTypeService;
    @Resource
    private ISysDeptService deptService;

    @Transactional
    @Override
    public List<DeptStatVo> deptGoodsReceiveStat(String m) {
        List<SysDictData> goodsStorageType = dictTypeService.selectDictDataByType("goods_storage_type");
        List<SysDept> sysDepts = deptService.selectChildrenDeptById(299L);
        List<DeptStatListVo> list = new ArrayList<>();
        Map<String, Double> mapTemp = new HashMap<>();
        goodsStorageType.forEach(dictData -> {
            mapTemp.put(dictData.getDictValue(), 0.0);
        });
        sysDepts.forEach(dept -> {
            list.add(new DeptStatListVo(dept.getId(),dept.getDeptName(), mapTemp));
        });

        Cursor<DeptStatVo> cursor = baseMapper.deptGoodsReceiveStat(m);
        cursor.forEach(item -> {
            list.forEach(stat -> {
                if (StringUtils.isNotBlank(item.getDeptId()) && item.getDeptId().equals(stat.getDeptId())) {
                    double newPrice = stat.getMap().get(item.getRecordType()) + item.getAverageUnitPrice();
                    stat.getMap().put(item.getRecordType(), newPrice);
                }
            });
        });
        return null;
    }
}
